# pylint: skip-file
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import numpy as np
import glob
import pickle
import os.path
import math

def load_data_img(mode="train"):
	data = np.load("data/chagas.dataset.lr.npy")
	labels = np.load("data/chagas.labels.lr.npy")
	if mode == "train":
		return data[0:2000] / 255.0, labels[0:2000]

	if mode == "test":
		return data[2000:] / 255.0, labels[2000:]
	
	return None



class _Batch(object):
	def __init__(self, dataset):
		self._size = 1 #number of elements that the batch should return. 
		self._ptr = 0 #a pointer to the begining of the current batch.
		self._elements = np.copy(dataset) #stores the indexes of the dataset that belong to the batch. 
		self._will_restart = False #true if all the elements of the dataset have been seen. 
		self.shuffle()

	@property
	def size(self):
		return self._size	
	@size.setter
	def size(self, value):
		v = int(value)
		if v < 1:
			raise ValueError("Size must be bigger than 0")
		self._size = v

	@property
	def will_restart(self):
		return self._will_restart

	def shuffle(self):
		perm = np.arange(len(self._elements))
		np.random.shuffle(perm)
		self._elements = self._elements[perm]
	
	def nextBatch(self):
		if self._will_restart:
			self.shuffle()
			self._ptr = 0
			self._will_restart = False		
		batch_end = self._ptr + self._size
		if batch_end >= self._elements.size:
			batch_end = self._elements.size
			self._will_restart = True		
		current_batch = self._elements[self._ptr: batch_end]
		self._ptr = batch_end
		return current_batch.astype(np.uint32)


class DataProvider(object):
	def __init__(self, reader):
		self._load_funct = reader
		self._samples, self._labels = None, None
		self._num_examples = 0
		self._num_epoch = 1
		self._epoch_indx = 0
		self._train_batch = None
		self._valid_batch = None
		self._test_batch = None		
		

	def loadSamples(self, **kwargs):
		self._samples, self._labels = self._load_funct(**kwargs)
		self._num_examples = len(self._samples)
		self._train_batch = _Batch(np.arange(len(self._samples)))


	#For saving and restoring batches
	def savePartition(self, save_path):
		np.save(save_path + '.tr.npy', self._train_batch._elements)
		np.save(save_path + '.te.npy', self._test_batch._elements)
		np.save(save_path + '.va.npy', self._valid_batch._elements)
	
	def loadPartition(self, load_path):
		aux_arr = np.load(load_path + '.tr.npy')
		self._train_batch = _Batch(aux_arr)
		aux_arr = np.load(load_path + '.te.npy')
		self._test_batch = _Batch(aux_arr)
		aux_arr = np.load(load_path + '.va.npy')
		self._valid_batch = _Batch(aux_arr)

	# partition the data stored inside _images (and corresponding groundtruth) into train, val and test
	# split = () a 3-element tuple defining the percentage of each set
	def partition(self, split):
		perm = np.arange(self._num_examples)
		np.random.shuffle(perm)
		train_idx = perm[0:math.floor(split[0]*self._num_examples)]
		val_idx = perm[math.floor(split[0]*self._num_examples):math.floor((split[0]+split[1])*self._num_examples)]
		test_idx = perm[math.floor((split[0]+split[1])*self._num_examples): self._num_examples]
		self._train_batch = _Batch(train_idx)
		self._valid_batch = _Batch(val_idx)
		self._test_batch = _Batch(test_idx)		
		

	def setBatchSize(self, size, dataset):
		if dataset.lower() == 'train':
			self._train_batch.size = size
			return
		
		if dataset.lower() == 'validation':
			self._valid_batch.size = size
			return

		if dataset.lower() == 'test':
			self._test_batch.size = size
			return
		print("........................DATA PROVIDER: Unknow option. Valid options are: train, validation or test.")		


	@property
	def num_epoch(self):
		return self._num_epoch	
	@num_epoch.setter
	def num_epoch(self, value):
		v = int(value)
		if v < 1:
			raise ValueError("Size must be at least 1")
		self._num_epoch = v

	
	def shouldStop(self):
		return self._num_epoch == self._epoch_indx

	
	def batchWillRestart(self, dataset):
		if dataset.lower() == 'train':			
			return self._train_batch.will_restart
		
		if dataset.lower() == 'validation':			
			return self._valid_batch.will_restart

		if dataset.lower() == 'test':			
			return self._test_batch.will_restart

		print("........................DATA PROVIDER: Unknow option. Valid options are: train, validation or test.")
		return None


	def nextBatch(self, dataset = 'train'):
		idx = None
		if dataset.lower() == 'train':			
			idx =  self._train_batch.nextBatch()
			if self._train_batch.will_restart:
				self._epoch_indx += 1
		
		if dataset.lower() == 'validation':			
			idx = self._valid_batch.nextBatch()

		if dataset.lower() == 'test':			
			idx = self._test_batch.nextBatch()
		
		if idx is None:
			print("........................DATA PROVIDER: Unknow option. Valid options are: train, validation or test.")
			return None, None

		return self._samples[idx], self._labels[idx]


	def next_batch(self, batch_size, set = 0, shuffle=True):
		idx = None
		if set == 0:			
			idx =  self._train_batch.nextBatch()
			if self._train_batch.will_restart:
				self._epoch_indx += 1
		
		if set == 1:			
			idx = self._valid_batch.nextBatch()

		if set == 2:			
			idx = self._test_batch.nextBatch()
		
		if idx is None:
			print("........................DATA PROVIDER: Unknow option. Valid options are: 0 (train), 1 (validation) or 2 (test).")
			return None, None

		return self._samples[idx], self._labels[idx]